//need to straighten head


//-*-c++-*-//
//Shawn Turner
//Aibo will walk straight until a barrier is detected by the IR sensor
//Aibo will then remain stopped until the barrier is no longer detected
//Good simple behavior example, CMU's RoboBits has a similar OPENR newbie
//assignment using the g-sensor (gravity sensor)
#ifndef INCLUDED_Box2_h_
#define INCLUDED_Box2_h_
#include "Behaviors/BehaviorBase.h" //necessary for all behavior
#include "SoundPlay/SoundManager.h" //necessary for any sound
#include "Motion/MotionManager.h" //for motions
#include "Motion/PostureMC.h" //unneccesary for this example
#include "Motion/WalkMC.h" //definition of the basic walk
#include "Shared/WorldState.h" //for state pointer
#include "Motion/MMAccessor.h" //to access active motions
#include "Shared/SharedObject.h" //always with motion
#include "Events/EventRouter.h" //for event handling (always)
#define IROORDist 900 //maximum 210 range (mm)
#define IRMinDist 600 //let's call this the minimum (mm)(was 300)
class Box2 : public BehaviorBase {
public:
/*
The constructor / destructor should be old news to you. You should implement
housekeeping up and down here.
*/
Box2():BehaviorBase(),start_walk(EventBase::buttonEGID,0,EventBase::activateETID,0), walker_id(MotionManager::invalid_MC_ID)
{
//do nothing
}
~Box2()
{
//if somehow were still running when this runs, we should be sure to turn it off
if(isActive())
DoStop();
}
// DoStart is typically where the setup logic / motion initializations occur
virtual void DoStart() {
//call superclass first for housekeeping:
BehaviorBase::DoStart();
/*
setup and start a motion command.
Details on this can be found in the Writing your first MotionCommand tutorial
on Tekkotsu.org . the addMotion method is the equivalent of turning on the motoin. In
this case, the WalkMC.
*/
walk_id=motman->addMotion(SharedObject<WalkMC>());
/*
any event that this behavior cares about needs to be registered with the event router.
This tells the event router to send us a message if it detects such an event.
In this case, I want to know if any sensor data has changed.
A good list of pre-defined trappable events can be found here:
http://www-2.cs.cmu.edu/~tekkotsu/dox/classEventBase.html
*/
erouter->addListener(this,start_walk); // subscribes to all sensor events
} //end DoStart
/*
DoStop basically undoes all the setup found in DoStart
Should be fairly straightforward
*/
virtual void DoStop() {
motman->removeMotion(walk_id); //remove the WalkMC motion tied to walk_id
erouter->forgetListener(this); // stops getting events (and timers, if we had any)
BehaviorBase::DoStop(); // run base class for consistency / cleanup
}
/*
processEvent really does all the work here. We want Aibo to walk as so long as there is nothing
directly in its path. Ive used the IR sensor set at an arbitrary minimum to detect an obstacle and
effectively stop walking
processEvent runs anytime an event we registered for is detected. Because we can register for
as many events as we like, its a good idea to use a switch or if statement to detect exactly which
event were handling, as well as if we somehow wound up with an event that we didnt think we
registered for (debugging)
*/
virtual void processEvent(const EventBase& event) {
		int remainingSteps;
		//get the current distance from the IR sensor
		cout<<"Processing Event";
		if(event==start_walk)
		{
			cout<<"Walking";
			MMAccessor<WalkMC> walk(walk_id);
			
			//update the walk on a dead ahead vector
			walk.mc()->setTargetVelocity(150.0, 0, 0,8);
			
			remainingSteps = walk.mc()->getRemainingSteps();
			while ( remainingSteps > 0 )
				remainingSteps = walk.mc()->getRemainingSteps();

			walk.mc()->setTargetVelocity(-150.0, 0, 0,8);
		}
		else
		//this should never happen, but can be real helpful when we forget to register correctly!
			cout << "Bad Event:" << event.getName() << endl;
	}
//return an approriate string representing the name. This too, is neccessary.
virtual std::string getName() const { return "Box2"; }
protected:
MotionManager::MC_ID walk_id; //create an MC_ID for my walk
EventBase start_walk; //event mask for starting walk
}; //end class
#endif