#ifndef INCLUDED_SampleLedBehavior_h_
#define INCLUDED_SampleLedBehavior_h_

#include "Behaviors/BehaviorBase.h"
#include "Motion/LedMC.h"
#include "Motion/MotionManager.h"
#include "Motion/MMAccessor.h"
#include "Shared/SharedObject.h"
#include "Events/EventRouter.h"

// Just a quick little demo behavior - when a button is pressed, this will
// light up the LED which happens to have the same index number as the
// button that was pressed

class SampleLedBehavior : public BehaviorBase {
	public:
		SampleLedBehavior() : BehaviorBase(), leds_id(MotionManager::invalid_MC_ID) {}
		
		virtual void DoStart() {
			//call superclass first for housekeeping:
			BehaviorBase::DoStart();
			
			//now do your code:
			std::cout<<"LED Start!";
			leds_id=motman->addMotion(SharedObject<LedMC>());    // creates an LED controller
			erouter->addListener(this,EventBase::buttonEGID); // subscribes to all button events
			}
		
		virtual void DoStop() {
			//do your code:
			motman->removeMotion(leds_id);    // removes the LED controller
			erouter->forgetListener(this); // stops getting events (and timers, if we had any)
			std::cout<<"LED End!";

			//but don't forget to call superclass:
			BehaviorBase::DoStop();
			}
		
		virtual void processEvent(const EventBase& event) {
			// to be more general, let's check that it's the right event:
			
			if(event.getGeneratorID()==EventBase::buttonEGID) {
				
				// if it's an activate or status event, turn the LED on, otherwise turn it off
				double value = (event.getTypeID()!=EventBase::deactivateETID)?1:0;
				
				// make a bitmask corresponding to the button's source ID
				//   LEDBitMask_t is just an unsigned int, defined in RobotInfo.h
				LEDBitMask_t bitmask = (1<<event.getSourceID());
				
				// "checks out" the motion primitive from MotionManager
				MMAccessor<LedMC> leds_mc(leds_id);
				
				// now send the bitmask and value to the LED controller
				leds_mc->set(bitmask,value);
				
				// notice that there's no "check in"
				// MMAccessor's destructor does this automatically
			} else {
				//should never happen
				cout << "Bad Event:" << event.getName() << endl;
			}
		}
		
		virtual std::string getName() const {
			// Name is used for menus, or debugging.  We've been using the convention
			return "SampleLedBehavior";
		}

protected:
	MotionManager::MC_ID leds_id; //id of LED controller
};
#endif 