#include "SimpleSoccer.h"
#include "Events/EventRouter.h"
#include "Events/VisionObjectEvent.h"
#include "Shared/WorldState.h"
#include "Motion/HeadPointerMC.h"
#include "Motion/WalkMC.h"
#include "Shared/WMclass.h"
#include "Shared/ProjectInterface.h"

//! Converts degrees to radians
inline double DtoR(double deg) { return (deg/180.0*M_PI); }

void SimpleSoccer::DoStart() {

	BehaviorBase::DoStart();
	//PlayerType
	player_type = GOALIE_ID;
	//previous Area
	previous_area = 999;
	//play_mode
	play_mode = PLAY_PAUSED;

	sndman->LoadFile("SoccerPr.wav");
	sndman->LoadFile("GoalieOn.wav");
	sndman->LoadFile("ForwardS.wav");
	sndman->LoadFile("ImGonnaS.wav");

	//sndman->PlayFile("SoccerPr.wav");

	headpointer_id = motman->addPersistentMotion(SharedObject<HeadPointerMC>());
	walker_id = motman->addPersistentMotion(SharedObject<WalkMC>());
	erouter->addListener(this,EventBase::visObjEGID,ProjectInterface::visPinkBallSID);
	erouter->addListener(this,EventBase::buttonEGID);
}

void SimpleSoccer::DoStop() {
	erouter->removeListener(this);
	motman->removeMotion(headpointer_id);
	motman->removeMotion(walker_id);
	sndman->ReleaseFile("SoccerPr.wav");
	sndman->ReleaseFile("GoalieOn.wav");
	sndman->ReleaseFile("ImGonnaS.wav");
	sndman->ReleaseFile("ForwardS.wav");
	BehaviorBase::DoStop();
}

//this could be cleaned up event-wise (only use a timer when out of view)
void SimpleSoccer::processEvent(const EventBase& event) {
	if (event==HBtn){
			cout<<"Head Button Pressed"<<endl;
			play_mode = PLAY_PAUSED;
			cout << "Paused" << endl;
			WalkMC * walker = (WalkMC*)motman->checkoutMotion(walker_id);
			walker->setTargetVelocity(0,0,0);			
			motman->checkinMotion(walker_id);

			switch (player_type){
				case GOALIE_ID:
					player_type = ATTACKER_ID;
					desired_area = ATTACKER_DESIRED_AREA;
					sndman->PlayFile("ForwardS.wav");
					break;
				case ATTACKER_ID:
					player_type = ATTACKER_ID2;
					desired_area = ATTACKER_DESIRED_AREA2;
					sndman->PlayFile("ImGonnaS.wav");
					break;
				case ATTACKER_ID2:
					player_type = GOALIE_ID;
					desired_area = GOALIE_DESIRED_AREA;
					sndman->PlayFile("GoalieOn.wav");
					break;
			}
	}

	if(event.getSourceID()==ChinButOffset){
		if(event.getTypeID()==EventBase::activateETID) {
			cout << "Chin Pressed" << endl;
			switch (play_mode){
				case PLAY_PAUSED:
					play_mode = PLAY_PLAY;
					cout << "PLAY" << endl;
					break;
				case PLAY_PLAY:
					play_mode = PLAY_PAUSED;
					cout << "Paused" << endl;
					break;
			}
		}

	}

	if (play_mode == PLAY_PLAY && event.getSourceID()==ProjectInterface::visPinkBallSID && (player_type == ATTACKER_ID || player_type == ATTACKER_ID2))
	{

		WMreg(chase_ball_behavior);
		WMvari_(float, horiz, 0, chase_ball_behavior);
		WMvari_(float, vert, 0, chase_ball_behavior);
		WMvari_(float, area, 0, chase_ball_behavior);

		if(event.getGeneratorID()==EventBase::visObjEGID && event.getTypeID()==EventBase::statusETID) {
			horiz=static_cast<const VisionObjectEvent*>(&event)->getCenterX();
			vert=static_cast<const VisionObjectEvent*>(&event)->getCenterY();
			area=static_cast<const VisionObjectEvent*>(&event)->getArea ();
		}
	
		double tilt=state->outputs[HeadOffset+TiltOffset]-vert*M_PI/7;
		double pan=state->outputs[HeadOffset+PanOffset]-horiz*M_PI/6;
		if(tilt<DtoR(-30))
			tilt=DtoR(-30);
		if(tilt>DtoR(40))
			tilt=DtoR(40);
		if(pan>DtoR(80))
			pan=DtoR(80);
		if(pan<DtoR(-80))
			pan=DtoR(-80);
		HeadPointerMC * headpointer= (HeadPointerMC*)motman->checkoutMotion(headpointer_id);
		headpointer->setJoints(tilt,pan,0);
		motman->checkinMotion(headpointer_id);

		cout << "Area: " <<  area << endl;

		WalkMC * walker = (WalkMC*)motman->checkoutMotion(walker_id);
		
		if ((area > desired_area) || (previous_area == 999) || (area >= previous_area)){
			//walk to ball
			cout << "Previous Area" <<  previous_area << endl;
			previous_area = area;
			if(state->outputs[HeadOffset+PanOffset]<-.05 || state->outputs[HeadOffset+PanOffset]>.05)
				walker->setTargetVelocity(100,0,state->outputs[HeadOffset+PanOffset]);
			else
				walker->setTargetVelocity(160,0,0);
			
			motman->checkinMotion(walker_id);
		}
		else {//stop and shoot
			walker->setTargetVelocity(0,0,0);
			motman->checkinMotion(walker_id);
			cout << "KICK!" << endl;
			
			switch (player_type){
				case ATTACKER_ID:
					SharedObject<MediumMotionSequenceMC>  s("/ms/data/motion/gu_front.mot");
					motman->addPrunableMotion(s);
					break;
				//case ATTACKER_ID2;
				//	SharedObject<MediumMotionSequenceMC>  r("/ms/data/motion/h_bump.mot");
				//	motman->addPrunableMotion(r);
				//	break;
			}
			//cout << "Load Time: " << timer.Age() << endl;
			previous_area = 999;
		}
	}
	if (play_mode == PLAY_PLAY && event.getSourceID()==ProjectInterface::visPinkBallSID && (player_type == GOALIE_ID))
	{

		WMreg(chase_ball_behavior);
		WMvari_(float, horiz, 0, chase_ball_behavior);
		WMvari_(float, vert, 0, chase_ball_behavior);
		WMvari_(float, area, 0, chase_ball_behavior);

		if(event.getGeneratorID()==EventBase::visObjEGID && event.getTypeID()==EventBase::statusETID) {
			horiz=static_cast<const VisionObjectEvent*>(&event)->getCenterX();
			vert=static_cast<const VisionObjectEvent*>(&event)->getCenterY();
			area=static_cast<const VisionObjectEvent*>(&event)->getArea ();
		}
	
		double tilt=state->outputs[HeadOffset+TiltOffset]-vert*M_PI/7;
		double pan=state->outputs[HeadOffset+PanOffset]-horiz*M_PI/6;
		if(tilt<DtoR(-70))
			tilt=DtoR(-70);
		if(tilt>DtoR(40))
			tilt=DtoR(40);
		if(pan>DtoR(80))
			pan=DtoR(80);
		if(pan<DtoR(-80))
			pan=DtoR(-80);
		HeadPointerMC * headpointer= (HeadPointerMC*)motman->checkoutMotion(headpointer_id);
		headpointer->setJoints(tilt,pan,0);
		motman->checkinMotion(headpointer_id);

		cout << "Goalie-Area: " <<  area << endl;

		WalkMC * walker = (WalkMC*)motman->checkoutMotion(walker_id);
		
		if ((area > desired_area) || (previous_area == 999) || (area >= previous_area)){
			previous_area = area;
			//walk to ball
			/*if(state->outputs[HeadOffset+PanOffset]<-.05 || state->outputs[HeadOffset+PanOffset]>.05)
				walker->setTargetVelocity(100,0,state->outputs[HeadOffset+PanOffset]);
			else
				walker->setTargetVelocity(0,0,state->outputs[HeadOffset+PanOffset]);
			
			motman->checkinMotion(walker_id);
			*/

		}
		else {//stop and shoot
			if (previous_area != 1000){
				walker->setTargetVelocity(0,0,0);
				motman->checkinMotion(walker_id);
				cout << "Stop!" << endl;
				SharedObject<MediumMotionSequenceMC>  g("/ms/data/motion/gu_side.mot");
				
				//cout << "Load Time: " << timer.Age() << endl;
				motman->addPrunableMotion(g);
			}
			previous_area = 1000;
		}
	}
}



