#include "SimpleSoccer.h"
#include "Events/EventRouter.h"
#include "Events/VisionObjectEvent.h"
#include "Shared/WorldState.h"
#include "Motion/HeadPointerMC.h"
#include "Motion/WalkMC.h"
#include "Shared/WMclass.h"
#include "Shared/ProjectInterface.h"

//! Converts degrees to radians
inline double DtoR(double deg) { return (deg/180.0*M_PI); }

void SimpleSoccer::DoStart() {

	BehaviorBase::DoStart();

	//INITIALIZE VARS
	//PlayerType... default to Attacker
	player_type = ATTACKER_ID;
	//previous Area... used to track how close the ball is... default to far away
	previous_area = 999;
	//play_mode.. default to paused.
	play_mode = PLAY_PAUSED;
	//Timer off
	timer_mode = TIMER_OFF;
	//timer Count = 0
	timer_counter = 0;

	//load sounds so that we can play them
	sndman->LoadFile("SoccerPr.wav");
	sndman->LoadFile("GoalieOn.wav");
	sndman->LoadFile("ForwardS.wav");
	sndman->LoadFile("ImGonnaS.wav");
	sndman->LoadFile("whimper.wav");

	//SharedObject<MediumMotionSequenceMC>  p("/ms/data/motion/pounce.pos");

	//prepare motion manager to move head and walk
	headpointer_id = motman->addPersistentMotion(SharedObject<HeadPointerMC>());
	walker_id = motman->addPersistentMotion(SharedObject<WalkMC>());
	//listen for events from vision (ID ball) and from buttons (switch player type/pause)
	erouter->addListener(this,EventBase::visObjEGID,ProjectInterface::visPinkBallSID);
	erouter->addListener(this,EventBase::visObjEGID,ProjectInterface::visYellowGoalSID);
	erouter->addListener(this,EventBase::buttonEGID);
	//1000 every second
	erouter->addTimer(this,0,1000,true);

}

void SimpleSoccer::DoStop() {
	erouter->removeListener(this);
	erouter->removeTimer(this);
	motman->removeMotion(headpointer_id);
	motman->removeMotion(walker_id);
	sndman->ReleaseFile("SoccerPr.wav");
	sndman->ReleaseFile("GoalieOn.wav");
	sndman->ReleaseFile("ImGonnaS.wav");
	sndman->ReleaseFile("ForwardS.wav");
	sndman->ReleaseFile("whimper.wav");
	BehaviorBase::DoStop();
}

//do something when an event is raised
//this could be cleaned up event-wise (only use a timer when out of view)
void SimpleSoccer::processEvent(const EventBase& event) {
	
	//test timer
	if(event.getGeneratorID()==EventBase::timerEGID) {
				//sndman->PlayFile("whimper.wav");
		if (timer_mode == TIMER_ON){
			cout<<"timer event"<<endl;
			timer_counter++;
		}
	}
	
	if (event==HBtn){//if it is the head button, switch player type and pause
			cout<<"Head Button Pressed"<<endl;
			play_mode = PLAY_PAUSED;
			cout << "Paused" << endl;
			
			//switch player and desired area (to trigger kick or defensive block)
			switch (player_type){
				case GOALIE_ID:
					player_type = ATTACKER_ID;
					desired_area = ATTACKER_DESIRED_AREA;
					sndman->PlayFile("ForwardS.wav");
					break;
				case ATTACKER_ID:
					player_type = GOALIE_ID;
					desired_area = GOALIE_DESIRED_AREA;
					sndman->PlayFile("GoalieOn.wav");
					break;
			}
	}
	
	//on press of chin button pause or unpause
	if(event.getSourceID()==ChinButOffset){
		if(event.getTypeID()==EventBase::activateETID) {
			cout << "Chin Pressed" << endl;
			switch (play_mode){
				case PLAY_PAUSED:
					play_mode = PLAY_PLAY;
					cout << "PLAY" << endl;
					break;
				case PLAY_PLAY:
					play_mode = PLAY_PAUSED;
					cout << "Paused" << endl;
					break;
			}
		}

	}
	
	//decide action based on mode and event
	if (play_mode == PLAY_PAUSED){
			//do nothing
			WalkMC * walker = (WalkMC*)motman->checkoutMotion(walker_id);
			walker->setTargetVelocity(0,0,0);		
			motman->checkinMotion(walker_id);
			HeadPointerMC * headpointer= (HeadPointerMC*)motman->checkoutMotion(headpointer_id);
			headpointer->setJoints(0,0,0);
			motman->checkinMotion(headpointer_id);
			cout<<"Stop Walk, Head Up"<<endl;
	}

	if (play_mode == PLAY_PLAY){	
			//determine starting mode based on player
			switch (player_type){
				case GOALIE_ID:
					play_mode = PLAY_PAUSED;//tbd
					break;
				case ATTACKER_ID:
					play_mode = PLAY_LOOK_FOR_BALL;
					break;
			}
	}

	if (play_mode == PLAY_LOOK_FOR_BALL){
		//if see ball, set go to ball.
		if (event.getSourceID()==ProjectInterface::visPinkBallSID){
			play_mode = PLAY_GO_TO_BALL;
			cout<<"BALL FOUND"<<endl;
		}
		else{
		//else 360 until see ball
		cout<<"Look for ball"<<endl;
		WalkMC * walker = (WalkMC*)motman->checkoutMotion(walker_id);
		walker->setTargetVelocity(12,100,39);//rotate pretty large circle with head in center		
		motman->checkinMotion(walker_id);
		}
	}

	if (play_mode == PLAY_GO_TO_BALL){
		//we see the ball, so lets go to it
		//stop when near ball (see distance) then set new state/mode
		if(event.getSourceID()==ProjectInterface::visPinkBallSID && event.getGeneratorID()==EventBase::visObjEGID && event.getTypeID()==EventBase::statusETID) {
			//some vars to hold chasing ball data
			WMreg(chase_ball_behavior);
			WMvari_(float, horiz, 0, chase_ball_behavior);
			WMvari_(float, vert, 0, chase_ball_behavior);
			WMvari_(float, area, 0, chase_ball_behavior);

			//get info on ball
			horiz=static_cast<const VisionObjectEvent*>(&event)->getCenterX();
			vert=static_cast<const VisionObjectEvent*>(&event)->getCenterY();
			area=static_cast<const VisionObjectEvent*>(&event)->getArea ();

			//set head to follow ball
			double tilt=state->outputs[HeadOffset+TiltOffset]-vert*M_PI/7;
			double pan=state->outputs[HeadOffset+PanOffset]-horiz*M_PI/6;
			if(tilt<DtoR(-30))
				tilt=DtoR(-30);
			if(tilt>DtoR(40))
				tilt=DtoR(40);
			if(pan>DtoR(80))
				pan=DtoR(80);
			if(pan<DtoR(-80))
				pan=DtoR(-80);
			HeadPointerMC * headpointer= (HeadPointerMC*)motman->checkoutMotion(headpointer_id);
			headpointer->setJoints(tilt,pan,0);
			motman->checkinMotion(headpointer_id);

			cout << "Area: " <<  area << "    Previous Area: " <<  previous_area << "    Desired Area: " << desired_area << endl;

			WalkMC * walker = (WalkMC*)motman->checkoutMotion(walker_id);
			
			//if just starting or still farther from ball
			if ((area > desired_area) || (previous_area == 999) || (area >= previous_area)){
				//walk to ball
				//cout << "Previous Area" <<  previous_area << endl;
				if (area > desired_area){previous_area = area;}
				if(state->outputs[HeadOffset+PanOffset]<-.05 || state->outputs[HeadOffset+PanOffset]>.05)
					walker->setTargetVelocity(100,0,state->outputs[HeadOffset+PanOffset]);
				else
					walker->setTargetVelocity(160,0,0);
				
				motman->checkinMotion(walker_id);
			}
			//else if we are close to the ball, change mode
			else if (area <= previous_area){
				play_mode = PLAY_NEAR_BALL_FIND_GOAL;
				cout << "Set PLAY_NEAR_BALL_FIND_GOAL" << endl;
			}
		}
	}
		
		/*case PLAY_NEAR_BALL_FIND_GOAL:
				//rotate around ball until goal is seen
				if (event.getSourceID()==ProjectInterface::visYellowGoalSID)
				{
					//test for goal recognition
					cout <<"GOAL IN VIEW" << endl;
				}
				break;
		case PLAY_BALL_GOAL_INLINE:
			break;
	*///close switch (play_mode)

}



