//-*-c++-*-
#ifndef INCLUDED_SimpleAttacker_h_
#define INCLUDED_SimpleAttacker_h_

#include "Behaviors/BehaviorBase.h"
#include "Motion/MotionManager.h"
#include "Behaviors/Controls/FileBrowserControl.h"
#include "Motion/MotionSequenceMC.h"
#include "Motion/EmergencyStopMC.h"
#include "Motion/LedMC.h"
#include "SoundPlay/SoundManager.h"
#include "Motion/MMAccessor.h"
#include "Shared/TimeET.h"
#include "Shared/Config.h"
#include <string>

//! A simple behavior to chase after any objects seen by the vision system
class SimpleAttacker : public BehaviorBase {
public:
	//!constructor
	SimpleAttacker()
		: BehaviorBase("SimpleAttacker"), headpointer_id(MotionManager::invalid_MC_ID), walker_id(MotionManager::invalid_MC_ID)
	{}
	//!destructor
	virtual ~SimpleAttacker() {}

	//! adds a headpointer and a walker, and a listens for vision events
	virtual void DoStart();

	//! removes motion commands and stops listening
	virtual void DoStop();

	//! sets the head to point at the object and sets the body to move where the head points
	virtual void processEvent(const EventBase& event);
			
	static std::string getClassDescription() { return "Follows ball with head and walks whereever the head is pointing"; }
	virtual std::string getDescription() const { return getClassDescription(); }

protected:
	int player_type;	//PlayerType
	const int GOALIE_ID; //const Goalie
	const int ATTACKER_ID;//conts Attacker
	float previous_area; //previous Area
	MotionManager::MC_ID headpointer_id; //!< a HeadPointerMC object
	MotionManager::MC_ID walker_id;      //!< a WalkMC object
};

/*! @file
 * @brief Describes SimpleAttacker, which runs around after whatever the dog sees
 * @author ejt (Creator)
 *
 * $Author: ejt $
 * $Name: tekkotsu-2_3 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $Date: 2004/11/11 01:45:35 $
 */

#endif
